@@ -60,8 +60,8 @@ namespace clx {
 		typedef CharT char_type;
 		typedef size_t size_type;
 		typedef std::basic_string<CharT, Traits> string_type;
-		typedef std::vector<string_type> container_type;
-		typedef std::map<string_type, container_type> map_type;
+		typedef std::vector<string_type> container;
+		typedef std::map<string_type, container> map_type;
 		typedef typename map_type::key_type key_type;
 		typedef typename map_type::mapped_type param_list;
 		
@@ -87,6 +87,7 @@ namespace clx {
 			for (int i = 1; i < argc; ++i) {
 				string_type tmp(argv[i]);
 				if (tmp.at(0) == identifier_ && tmp.size() > 1 &&
+					//!(tmp.at(1) >= zero && tmp.at(1) <= nine)) {
 					!(tmp.at(1) >= zero && tmp.at(1) <= nine)) {
 					eoh = true;
 					if (tmp.at(1) != identifier_) {
@@ -171,7 +172,7 @@ namespace clx {
 		
 		char_type identifier_;
 		string_type program_;
-		container_type head_;
+		container head_;
 		map_type v_;
 		
 		bool set_param(const key_type& x, const string_type& s) {
@@ -180,7 +181,7 @@ namespace clx {
 			
 			iterator pos = v_.find(x);
 			if (pos == v_.end()) {
-				container_type v;
+				container v;
 				if (!s.empty()) v.push_back(s);
 				std::pair<iterator, bool> r = v_.insert(value_type(x, v));
 				return r.second;
