@@ -29,7 +29,7 @@
  *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *  Last-modified: Sun 26 Oct 2008 17:14:00 JST
+ *  Last-modified: Thu 25 Dec 2008 07:46:00 JST
  */
 /* ------------------------------------------------------------------------- */
 #ifndef CLX_POP3_H
@@ -44,42 +44,6 @@
 #include "md5.h"
 
 namespace clx {
-	namespace detail {
-		/* ----------------------------------------------------------------- */
-		/*
-		 *  recvmail
-		 *
-		 *  The recvmail class is the Functor class for the rerieve method
-		 *  of basic_pop3 class.
-		 */
-		/* ----------------------------------------------------------------- */
-		class recvmail {
-		public:
-			typedef char char_type;
-			typedef std::basic_string<char_type> string_type;
-			
-			recvmail() : message_() {}
-			virtual ~recvmail() {}
-			
-			template <class InputStream>
-			void operator()(InputStream& sin) {
-				string_type buf;
-				std::basic_stringstream<char_type> dest;
-				while (std::getline(sin, buf)) {
-					chomp(buf);
-					if (buf[0] == '.') break;
-					dest << buf << std::endl;
-				}
-				message_ = dest.str();
-			}
-			
-			const string_type& str() const { return message_; }
-			
-		private:
-			string_type message_;
-		};
-	}
-	
 	/* --------------------------------------------------------------------- */
 	//  pop3_error
 	/* --------------------------------------------------------------------- */
@@ -90,18 +54,17 @@ namespace clx {
 		typedef std::stringstream stream_type;
 		
 		explicit pop3_error(const string_type& command, const string_type& what_arg) :
-			std::runtime_error(what_arg), command_(command) {}
+			std::runtime_error(what_arg), message_(command) {}
 		
 		virtual ~pop3_error() throw() {}
 		
 		virtual const char_type* what() {
-			string_type message;
-			message = command_ + ": " + std::runtime_error::what();
-			return message.c_str();
+			message_ = message_ + ": " + std::runtime_error::what();
+			return message_.c_str();
 		}
 		
 	private:
-		string_type command_;
+		string_type message_;
 	};
 	
 	/* --------------------------------------------------------------------- */
@@ -128,42 +91,12 @@ namespace clx {
 		
 		explicit basic_pop3(const string_type& host, int port = 110) :
 			socket_type(), host_(host), port_(port), timestamp_() {
-			try {
-				this->start();
-			}
-			catch (pop3_error& e) {
-				this->finish();
-				(*this) = basic_pop3();
-				throw e;
-			}
-			catch (socket_error& e) {
-				(*this) = basic_pop3();
-				throw e;
-			}
-			catch (std::runtime_error& e) {
-				(*this) = basic_pop3();
-				throw e;
-			}
+			this->start();
 		}
 		
 		explicit basic_pop3(const char_type* host, int port = 110) :
 			socket_type(), host_(host), port_(port), timestamp_() {
-			try {
-				this->start();
-			}
-			catch (pop3_error& e) {
-				this->finish();
-				(*this) = basic_pop3();
-				throw e;
-			}
-			catch (socket_error& e) {
-				(*this) = basic_pop3();
-				throw e;
-			}
-			catch (std::runtime_error& e) {
-				(*this) = basic_pop3();
-				throw e;
-			}
+			this->start();
 		}
 		
 		virtual ~basic_pop3() throw() { this->finish(); }
@@ -195,12 +128,8 @@ namespace clx {
 		
 		void finish() {
 			if (this->is_open()) {
-				basic_sockbuf<socket_type, char_type> sbuf(*this);
-				std::basic_iostream<char_type> ss(&sbuf);
-				
-				ss << "QUIT\r\n";
 				string_type buf;
-				this->xresponse(ss, buf);
+				this->xcommand("QUIT", buf);
 				this->close();
 			}
 		}
@@ -211,13 +140,9 @@ namespace clx {
 			std::basic_iostream<char_type> ss(&sbuf);
 			
 			// basic login has 2 steps
-			string_type buf = "USER " + id + "\r\n";
-			ss << buf;
-			if (!this->xresponse(ss, buf)) throw pop3_error("USER", buf);
-			
-			buf = "PASS " + pass + "\r\n";
-			ss << buf;
-			if (!this->xresponse(ss, buf)) throw pop3_error("PASS", buf);
+			string_type buf;
+			if (!this->xcommand(ss, "USER " + id, buf)) throw pop3_error("USER", buf);
+			if (!this->xcommand(ss, "PASS " + pass, buf)) throw pop3_error("PASS", buf);
 			
 			return *this;
 		}
@@ -230,14 +155,11 @@ namespace clx {
 		
 		basic_pop3& apop(const string_type& id, const string_type& pass) {
 			if (timestamp_.empty()) throw pop3_error("APOP", "not supported");
-			basic_sockbuf<socket_type, char_type> sbuf(*this);
-			std::basic_iostream<char_type> ss(&sbuf);
-			
 			string_type buf = timestamp_ + pass;
 			md5 digest(buf);
-			buf = "APOP " + id + " " + digest.to_string() + "\r\n";
-			ss << buf;
-			if (!this->xresponse(ss, buf)) throw pop3_error("APOP", buf);
+			if (!this->xcommand("APOP " + id + " " + digest.to_string(), buf)) {
+				throw pop3_error("APOP", buf);
+			}
 			
 			return *this;
 		}
@@ -249,13 +171,8 @@ namespace clx {
 		}
 		
 		string_type status() {
-			basic_sockbuf<socket_type, char_type> sbuf(*this);
-			std::basic_iostream<char_type> ss(&sbuf);
-			
-			string_type buf = "STAT\r\n";
-			ss << buf;
-			if (!this->xresponse(ss, buf)) throw pop3_error("STAT", buf);
-			
+			string_type buf;
+			if (!this->xcommand("STAT", buf)) throw pop3_error("STAT", buf);
 			return buf.substr(4);
 		}
 		
@@ -263,9 +180,8 @@ namespace clx {
 			basic_sockbuf<socket_type, char_type> sbuf(*this);
 			std::basic_iostream<char_type> ss(&sbuf);
 			
-			string_type buf = "LIST\r\n";
-			ss << buf;
-			if (!this->xresponse(ss, buf)) throw pop3_error("LIST", buf);
+			string_type buf;
+			if (!this->xcommand(ss, "LIST", buf)) throw pop3_error("LIST", buf);
 			
 			list_type dest;
 			while (std::getline(ss, buf)) {
@@ -278,13 +194,10 @@ namespace clx {
 		}
 		
 		string_type list(int n) {
-			basic_sockbuf<socket_type, char_type> sbuf(*this);
-			std::basic_iostream<char_type> ss(&sbuf);
-			
-			string_type buf = "LIST " + lexical_cast<string_type>(n) + "\r\n";
-			ss << buf;
-			if (!this->xresponse(ss, buf)) throw pop3_error("LIST", buf);
-			
+			string_type buf;
+			if (!this->xcommand("LIST " + lexical_cast<string_type>(n), buf)) {
+				throw pop3_error("LIST", buf);
+			}
 			return buf.substr(4);
 		}
 		
@@ -292,9 +205,8 @@ namespace clx {
 			basic_sockbuf<socket_type, char_type> sbuf(*this);
 			std::basic_iostream<char_type> ss(&sbuf);
 			
-			string_type buf = "UIDL\r\n";
-			ss << buf;
-			if (!this->xresponse(ss, buf)) throw pop3_error("UIDL", buf);
+			string_type buf;
+			if (!this->xcommand(ss, "UIDL", buf)) throw pop3_error("UIDL", buf);
 			
 			list_type dest;
 			while (std::getline(ss, buf)) {
@@ -307,13 +219,10 @@ namespace clx {
 		}
 		
 		string_type uid_list(int n) {
-			basic_sockbuf<socket_type, char_type> sbuf(*this);
-			std::basic_iostream<char_type> ss(&sbuf);
-			
-			string_type buf = "UIDL " + lexical_cast<string_type>(n) + "\r\n";
-			ss << buf;
-			if (!this->xresponse(ss, buf)) throw pop3_error("UIDL", buf);
-			
+			string_type buf;
+			if (!this->xcommand("UIDL " + lexical_cast<string_type>(n), buf)) {
+				throw pop3_error("UIDL", buf);
+			}
 			return buf.substr(4);
 		}
 		
@@ -322,18 +231,32 @@ namespace clx {
 			basic_sockbuf<socket_type, char_type> sbuf(*this);
 			std::basic_iostream<char_type> ss(&sbuf);
 			
-			string_type buf = "RETR " + lexical_cast<string_type>(n) + "\r\n";
-			ss << buf;
-			if (!this->xresponse(ss, buf)) throw pop3_error("RETR", buf);
+			string_type buf;
+			if (!this->xcommand(ss, "RETR " + lexical_cast<string_type>(n), buf)) {
+				throw pop3_error("RETR", buf);
+			}
 			
 			f(ss);
 			return *this;
 		}
 		
 		string_type retrieve(int n) {
-			detail::recvmail receiver;
-			this->retrieve(n, receiver);
-			return receiver.str();
+			basic_sockbuf<socket_type, char_type> sbuf(*this);
+			std::basic_iostream<char_type> ss(&sbuf);
+			
+			string_type buf;
+			if (!this->xcommand(ss, "RETR " + lexical_cast<string_type>(n), buf)) {
+				throw pop3_error("RETR", buf);
+			}
+			
+			std::basic_stringstream<char_type> dest;
+			while (std::getline(ss, buf)) {
+				chomp(buf);
+				if (buf[0] == '.') break;
+				dest << buf << std::endl;
+			}
+			
+			return dest.str();
 		}
 		
 		string_type top(int n, int line) {
@@ -342,8 +265,7 @@ namespace clx {
 			
 			string_type buf = "TOP " + lexical_cast<string_type>(n) + " " +
 				lexical_cast<string_type>(line) + "\r\n";
-			ss << buf;
-			if (!this->xresponse(ss, buf)) throw pop3_error("TOP", buf);
+			if (!this->xcommand(ss, buf, buf)) throw pop3_error("TOP", buf);
 			
 			std::basic_stringstream<char_type> dest;
 			while (std::getline(ss, buf)) {
@@ -356,32 +278,20 @@ namespace clx {
 		}
 		
 		basic_pop3& erase(int n) {
-			basic_sockbuf<socket_type, char_type> sbuf(*this);
-			std::basic_iostream<char_type> ss(&sbuf);
-			
-			string_type buf = "DELE " + lexical_cast<string_type>(n) + "\r\n";
-			ss << buf;
-			this->xresponse(ss, buf);
+			string_type buf;
+			this->xcommand("DELE " + lexical_cast<string_type>(n), buf);
 			return *this;
 		}
 		
 		basic_pop3& reset() {
-			basic_sockbuf<socket_type, char_type> sbuf(*this);
-			std::basic_iostream<char_type> ss(&sbuf);
-			
-			string_type buf = "RSET\r\n";
-			ss << buf;
-			this->xresponse(ss, buf);
+			string_type buf;
+			this->xcommand("RSET", buf);
 			return *this;
 		}
 		
 		basic_pop3& noop() {
-			basic_sockbuf<socket_type, char_type> sbuf(*this);
-			std::basic_iostream<char_type> ss(&sbuf);
-			
-			string_type buf = "NOOP\r\n";
-			ss << buf;
-			this->xresponse(ss, buf);
+			string_type buf;
+			this->xcommand("NOOP", buf);
 			return *this;
 		}
 		
@@ -400,6 +310,18 @@ namespace clx {
 			if (dest.empty() || dest.find("-ERR") != string_type::npos) return false;
 			return true;
 		}
+		
+		template <class IOStream>
+		bool xcommand(IOStream& ss, const string_type& cmd, string_type& dest) {
+			ss << (cmd + "\r\n");
+			return this->xresponse(ss, dest);
+		}
+		
+		bool xcommand(const string_type& cmd, string_type& dest) {
+			basic_sockbuf<socket_type, char_type> sbuf(*this);
+			std::basic_iostream<char_type> ss(&sbuf);
+			return this->xcommand(ss, cmd, dest);
+		}
 	};
 	
 	typedef basic_pop3<tcp::socket> pop3;
