@@ -2,7 +2,7 @@
 /*
  *  sockbuf.h
  *
- *  Copyright (c) 2004 - 2008, clown. All rights reserved.
+ *  Copyright (c) 2004 - 2010, clown. All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
  *  modification, are permitted provided that the following conditions
@@ -29,7 +29,7 @@
  *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *  Last-modified: Wed 03 Sep 2008 20:24:00 JST
+ *  Last-modified: Wed 13 Jan 2010 05:36:00 JST
  */
 /* ------------------------------------------------------------------------- */
 #ifndef CLX_SOCKBUF_H
@@ -49,6 +49,8 @@ namespace clx {
 		class Traits = std::char_traits<CharT>
 	>
 	class basic_sockbuf : public std::basic_streambuf<CharT, Traits> {
+	private:
+		typedef std::basic_streambuf<CharT, Traits> super;
 	public:
 		typedef Socket socket_type;
 		typedef CharT char_type;
@@ -60,14 +62,14 @@ namespace clx {
 		enum { npback = 8, nbuf = 65536 };
 		
 		// constructor and destructor
-		explicit basic_sockbuf(size_type n = nbuf) :
-			super(), sock_(), get_buffer_(n) {
-			this->setg(&get_buffer_[0], &get_buffer_[npback], &get_buffer_[npback]);
+		explicit basic_sockbuf(size_type n) :
+			super(), sock_(), buffer_(n) {
+			this->setg(&buffer_.at(0), &buffer_.at(npback), &buffer_.at(npback));
 		}
 		
 		explicit basic_sockbuf(socket_type& s, size_type n = nbuf) :
-			super(), sock_(&s), get_buffer_(n) {
-			this->setg(&get_buffer_[0], &get_buffer_[npback], &get_buffer_[npback]);
+			super(), sock_(&s), buffer_(n) {
+			this->setg(&buffer_.at(0), &buffer_.at(npback), &buffer_.at(npback));
 		}
 		
 		virtual ~basic_sockbuf() throw() {}
@@ -100,24 +102,30 @@ namespace clx {
 				// copy the previous read string to the putback field
 				int n = static_cast<int>(this->gptr() - this->eback());
 				if (n > npback) n = npback;
-				std::memcpy(&get_buffer_[npback-n], this->gptr() - n, n);
+				std::memcpy(&buffer_.at(npback - n), this->gptr() - n, n);
 				
-				int byte = static_cast<int>((get_buffer_.size() - npback) * sizeof(char_type));
-				int l = sock_->recv((char*)&get_buffer_[npback], byte - 1);
+				int byte = static_cast<int>((buffer_.size() - npback) * sizeof(char_type));
+				int l = sock_->recv(reinterpret_cast<char*>(&buffer_.at(npback)), byte);
 				if (l <= 0) return traits::eof();
-				this->setg(&get_buffer_[npback-n], &get_buffer_[npback], &get_buffer_[npback + l]);
+				this->setg(&buffer_.at(npback - n), &buffer_.at(npback), &buffer_.at(npback + l));
 				return traits::to_int_type(*this->gptr());
 			}
 			else return traits::eof();
 		}
 		
-		virtual int sync() { return this->overflow(traits::eof()); }
+		virtual int_type uflow() {
+			int_type c = this->underflow();
+			if (!traits::eq_int_type(c, traits::eof())) this->gbump(1);
+			return c;
+		}
+		
+		virtual int sync() {
+			return this->overflow(traits::eof());
+		}
 		
 	private:
-		typedef std::basic_streambuf<CharT, Traits> super;
-		
 		socket_type* sock_;
-		container get_buffer_;
+		container buffer_;
 	};
 };
 
