@@ -2,7 +2,7 @@
 /*
  *  vstream.h
  *
- *  Copyright (c) 2004 - 2009, clown. All rights reserved.
+ *  Copyright (c) 2004 - 2010, clown. All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
  *  modification, are permitted provided that the following conditions
@@ -29,7 +29,7 @@
  *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *  Last-modified: Fri 25 Dec 2009 15:45:00 JST
+ *  Last-modified: Wed 02 Jun 2010 05:01:00 JST
  */
 /* ------------------------------------------------------------------------- */
 #ifndef CLX_VSTREAM_H
@@ -66,8 +66,8 @@ namespace clx {
 		template <class Container>
 		explicit basic_ivstreambuf(Container& v) :
 			std::basic_streambuf<CharT, Traits>(),
-			begin_(reinterpret_cast<iterator>(&(*begin(v)))),
-			end_(reinterpret_cast<iterator>(&(*end(v)))) {
+			begin_(reinterpret_cast<iterator>(&(*clx::begin(v)))),
+			end_(reinterpret_cast<iterator>(&(*clx::end(v)))) {
 			this->setg(begin_, begin_, end_);
 		}
 		
@@ -83,7 +83,7 @@ namespace clx {
 				else if (way == std::ios_base::end) pos = last;
 				pos += pos_type(off);
 				if (pos < 0 || pos > last) return pos_type(-1);
-				this->setg(begin_, begin_ + pos, end_);
+				this->setg(begin_, begin_ + static_cast<int>(pos), end_);
 				return pos;
 			}
 			return pos_type(-1);
@@ -93,7 +93,7 @@ namespace clx {
 			if (which & std::ios_base::in) {
 				pos_type last = pos_type(end_ - begin_);
 				if (pos < 0 || pos > last) return pos_type(-1);
-				this->setg(begin_, begin_ + pos, end_);
+				this->setg(begin_, begin_ + static_cast<int>(pos), end_);
 				return pos;
 			}
 			return pos_type(-1);
